@@ -44,7 +44,11 @@
 */
 
 #ifndef gmic_version
-#define gmic_version 203
+#define gmic_version 218
+
+#ifndef gmic_pixel_type
+#define gmic_pixel_type float
+#endif
 
 #include <cstdio>
 #include <cstring>
@@ -161,16 +165,29 @@ namespace cimg_library {
 #endif
 #define cimg_plugin "gmic.cpp"
 
+// #ifdef cimg_use_abort
+// static struct cimg_is_abort {
+//   bool value, *ptr;
+//   cimg_is_abort():value(false),ptr(&value) {}
+// } _cimg_is_abort;
+// #define cimg_abort_test if (*_cimg_is_abort.ptr) throw CImgAbortException()
+// #endif // #ifdef cimg_use_abort
+
 #ifdef cimg_use_abort
-static struct cimg_is_abort {
-  bool value, *ptr;
-  cimg_is_abort():value(false),ptr(&value) {}
-} _cimg_is_abort;
-#define cimg_abort_test() if (*_cimg_is_abort.ptr) throw CImgAbortException()
+inline bool *gmic_abort_ptr(bool *const p_is_abort);
+#define cimg_abort_init bool *const gmic_is_abort = ::gmic_abort_ptr(0)
+#define cimg_abort_test if (*gmic_is_abort) throw CImgAbortException()
 #endif // #ifdef cimg_use_abort
+
+inline double gmic_mp_ext(char *const str, void *const p_list);
+#define cimg_mp_ext_function(str) return ::gmic_mp_ext(str._data,&mp.listout)
+
 #ifndef cimg_display
 #define cimg_display 0
 #endif // #ifndef cimg_display
+#ifndef cimg_appname
+#define cimg_appname "gmic"
+#endif // #ifndef cimg_appname
 #include "./CImg.h"
 
 #if cimg_OS==2
@@ -180,6 +197,7 @@ static struct cimg_is_abort {
 #elif cimg_OS==1
 #include <cerrno>
 #include <sys/resource.h>
+#include <sys/syscall.h>
 #include <signal.h>
 #endif // #if cimg_OS==2
 
@@ -228,6 +246,8 @@ struct gmic {
 
   // Functions below should be considered as *private*, and should not be
   // used in user's code.
+  template<typename T>
+  static bool search_sorted(const char *const str, const T& list, const unsigned int length, unsigned int &out_ind);
   static int _levenshtein(const char *const s, const char *const t,
                           gmic_image<int>& d, const int i, const int j);
   static int levenshtein(const char *const s, const char *const t);
@@ -239,6 +259,8 @@ struct gmic {
   static char *strreplace_bw(char *const str);
   static unsigned int strescape(const char *const str, char *const res);
   static const gmic_image<char>& decompress_stdlib();
+  static double mp_ext(char *const str, void *const p_list);
+  static bool *abort_ptr(bool *const p_is_abort);
 
   template<typename T>
   void _gmic(const char *const commands_line,
@@ -250,8 +272,10 @@ struct gmic {
                            const char operation='=',
                            const unsigned int *const variables_sizes=0);
 
-  gmic& add_commands(const char *const data_commands, const char *const commands_file=0);
-  gmic& add_commands(std::FILE *const file, const char *const filename=0);
+  gmic& add_commands(const char *const data_commands, const char *const commands_file=0,
+                     unsigned int *count_new=0, unsigned int *count_replaced=0);
+  gmic& add_commands(std::FILE *const file, const char *const filename=0,
+                     unsigned int *count_new=0, unsigned int *count_replaced=0);
 
   gmic_image<char> callstack2string(const bool is_debug=false) const;
   gmic_image<char> callstack2string(const gmic_image<unsigned int>& callstack_selection,
@@ -350,7 +374,10 @@ struct gmic {
              const gmic_image<unsigned int> *const command_selection);
 
   // Class variables.
+  static const char *builtin_commands_names[];
+  static gmic_image<int> builtin_commands_inds;
   static gmic_image<char> stdlib;
+  static gmic_list<void*> list_p_is_abort;
 
   gmic_list<char> *const commands, *const commands_names, *const commands_has_arguments,
     *const _variables, *const _variables_names, **const variables, **const variables_names,
@@ -358,15 +385,15 @@ struct gmic {
   gmic_image<unsigned int> dowhiles, fordones, repeatdones;
   gmic_image<unsigned char> light3d;
   gmic_image<char> status;
-  void *display_window;
+  void *display_windows;
 
   float focale3d, light3d_x, light3d_y, light3d_z, specular_lightness3d, specular_shininess3d, _progress, *progress;
   unsigned long reference_time;
   unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages, debug_filename, debug_line, cimg_exception_mode;
-  int verbosity, render3d, renderd3d;
-  bool is_released, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info, check_elif;
+  int verbosity,render3d, renderd3d;
+  bool is_released, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info, check_elif,
+    _is_abort, *is_abort, is_abort_thread;
   const char *starting_commands_line;
-  bool _is_abort, *is_abort, is_abort_thread;
 };
 
 // Class 'gmic_exception'.
@@ -398,6 +425,9 @@ struct gmic_exception {
   }
 };
 
+inline double gmic_mp_ext(char *const str, void *const p_list) { return gmic::mp_ext(str,p_list); }
+inline bool *gmic_abort_ptr(bool *const p_is_abort) { return gmic::abort_ptr(p_is_abort); }
+
 #endif // #ifndef gmic_version
 
 // Local Variables:
